@@ -17,11 +17,13 @@
     You should have received a copy of the GNU General Public License
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
-from gi.repository import Gio, Gtk, GLib, Gdk
+from gi.repository import Gio, Gtk
 
 import copy
 
+import animatedvalue
 import appconsts
+import callbackbridge
 import guipopover
 from editorstate import current_sequence
 import mltfilters
@@ -35,8 +37,6 @@ VOLUME_KF_EDIT = 0
 BRIGHTNESS_KF_EDIT = 1
 PARAM_KF_EDIT = 2
 
-select_clip_func = None
-
 _clip_popover = None
 _clip_menu = None
 _audio_submenu = None 
@@ -48,6 +48,7 @@ _tools_submenu = None
 _edit_bottom_section = None
 _compositor_section = None
 _title_section = None
+_generator_sub_menu = None
 _generator_section = None
 
 _audio_clip_popover = None
@@ -63,6 +64,7 @@ _blank_clip_menu = None
 _multi_clip_popover = None
 _multi_clip_menu = None
 _multi_audio_section = None
+_multi_sync_section = None
 
 _compositor_popover = None
 _compositor_menu = None
@@ -82,7 +84,7 @@ _kf_select_popover = None
 _kf_select_menu = None
  
  
-# -------------------------------------------------- menuitems builder fuctions
+# -------------------------------------------------- menuitems builder functions
 def add_menu_action(menu, label, item_id, data, callback, active=True, app=None):
     return guipopover.add_menu_action(menu, label, item_id, data, callback, active, app)
 
@@ -99,7 +101,7 @@ def add_menu_action_all_items_radio(menu, items_data, item_id, selected_index, c
 # -------------------------------------------------- clip menus
 def clip_popover_menu_show(widget, clip, track, x, y, callback):
     if clip.is_blanck_clip:
-        select_clip_func(track.id, track.clips.index(clip))
+        callbackbridge.movemodes_select_clip(track.id, track.clips.index(clip))
         blank_clip_popover_menu_show(widget, clip, track, x, y, callback)
         return
 
@@ -109,7 +111,7 @@ def clip_popover_menu_show(widget, clip, track, x, y, callback):
 
     global _clip_popover, _clip_menu, _audio_submenu, _markers_submenu, \
     _markers_submenu_static_items, _reload_section, _edit_actions_menu, \
-    _tools_submenu, _title_section, _compositor_section, _generator_section
+    _tools_submenu, _title_section, _compositor_section, _generator_section, _generator_sub_menu
 
     if _clip_menu == None:
         _clip_menu = guipopover.menu_clear_or_create(_clip_menu)
@@ -125,6 +127,8 @@ def clip_popover_menu_show(widget, clip, track, x, y, callback):
         _clip_menu.append_section(None, audio_section)
 
         _generator_section = Gio.Menu.new()
+        _generator_sub_menu = Gio.Menu.new()
+        _fill_generator_render_section(_generator_sub_menu, clip, callback)
         _fill_generator_section(_generator_section, clip, callback)
         _clip_menu.append_section(None, _generator_section)
         
@@ -150,27 +154,32 @@ def clip_popover_menu_show(widget, clip, track, x, y, callback):
         _clip_menu.append_section(None, _reload_section)
 
         edit_section = Gio.Menu.new()
-        select_menu = Gio.Menu.new()
-        _fill_select_menu(select_menu, callback)
-        edit_section.append_submenu(_("Select"), select_menu)
         _edit_actions_menu = Gio.Menu.new()
         _fill_edit_actions_menu(_edit_actions_menu, clip, track, callback)
         edit_section.append_submenu(_("Edit"), _edit_actions_menu)
+        select_menu = Gio.Menu.new()
+        _fill_select_menu(select_menu, callback)
+        edit_section.append_submenu(_("Select"), select_menu)
         _tools_submenu = Gio.Menu.new()
         _fill_tool_integration_menu(_tools_submenu, clip, callback)
-        edit_section.append_submenu(_("Export To Tool"), _tools_submenu)
+        edit_section.append_submenu(_("Slow Motion"), _tools_submenu)
         _clip_menu.append_section(None, edit_section)
 
         filters_section = Gio.Menu.new()
         add_filter_menu = Gio.Menu.new()
         _fill_filters_menus(add_filter_menu, callback, "add_filter", "clipmenu.addfilter.")
         filters_section.append_submenu(_("Add Filter"), add_filter_menu)
-        clone_sub_menu = Gio.Menu.new()
-        _fill_clone_filters_menu(clone_sub_menu, callback, False, False)
-        filters_section.append_submenu(_("Clone Filter"), clone_sub_menu)
         add_menu_action(filters_section, _("Clear Filters"), "clipmenu.clearfilters",  ("clear_filters", None), callback)
         _clip_menu.append_section(None, filters_section)
 
+        filters_copy_section = Gio.Menu.new()
+        clone_sub_menu = Gio.Menu.new()
+        _fill_clone_filters_menu(clone_sub_menu, callback, False, False)
+        filters_copy_section.append_submenu(_("Clone Filter"), clone_sub_menu)
+        add_menu_action(filters_copy_section, _("Copy Filters"), "clipmenu.copyfilters",  ("copy_filters", None), callback)
+        add_menu_action(filters_copy_section, _("Paste Filters"), "clipmenu.pastefilters",  ("paste_filters", None), callback)
+        _clip_menu.append_section(None, filters_copy_section)
+
         _title_section = Gio.Menu.new()
         _fill_title_section(_title_section, clip, callback)
         _clip_menu.append_section(None, _title_section)
@@ -202,15 +211,18 @@ def clip_popover_menu_show(widget, clip, track, x, y, callback):
         guipopover.menu_clear_or_create(_title_section)
         _fill_title_section(_title_section, clip, callback)
 
+        guipopover.menu_clear_or_create(_generator_sub_menu)
+        _fill_generator_render_section(_generator_sub_menu, clip, callback)
+
         guipopover.menu_clear_or_create(_generator_section)
         _fill_generator_section(_generator_section, clip, callback)
-
+        
     rect = guipopover.create_rect(x, y)
     _clip_popover = guipopover.new_mouse_popover(widget, _clip_menu, rect, Gtk.PositionType.TOP)
 
 def audio_clip_popover_menu_show(widget, clip, track, x, y, callback):
     if clip.is_blanck_clip:
-        select_clip_func(track.id, track.clips.index(clip))
+        callbackbridge.movemodes_select_clip(track.id, track.clips.index(clip))
         blank_clip_popover_menu_show(widget, clip, track, x, y, callback)
         return
         
@@ -263,6 +275,7 @@ def audio_clip_popover_menu_show(widget, clip, track, x, y, callback):
         _fill_clone_filters_menu(clone_sub_menu, callback, False, True)
         filters_section.append_submenu(_("Clone Filter"), clone_sub_menu)
         add_menu_action(filters_section, _("Clear Filters"), "audioclipmenu.clearfilters",  ("clear_filters", None), callback)
+        add_menu_action(filters_section, _("Paste Filters"), "audioclipmenu.pastefilters",  ("paste_filters", None), callback)
         _audio_clip_menu.append_section(None, filters_section)
 
         edit_bottom_section = Gio.Menu.new()
@@ -305,8 +318,8 @@ def multi_clip_popover_menu_show(widget, clip, track, x, y, callback):
     if hasattr(clip, "rendered_type"):
         transition_popover_menu_show(widget, clip, track, x, y, callback)
         return
-        
-    global _multi_clip_popover, _multi_clip_menu, _multi_audio_section
+
+    global _multi_clip_popover, _multi_clip_menu, _multi_audio_section, _multi_sync_section
 
     if _multi_clip_menu == None:
         _multi_clip_menu = guipopover.menu_clear_or_create(_multi_clip_menu)
@@ -315,29 +328,41 @@ def multi_clip_popover_menu_show(widget, clip, track, x, y, callback):
         _fill_multi_audio_section(_multi_audio_section, clip, track, callback)
         _multi_clip_menu.append_section(None, _multi_audio_section)
 
+        _multi_sync_section = Gio.Menu.new()
+        _fill_multi_sync_section(_multi_sync_section, clip, track, callback)
+        _multi_clip_menu.append_section(None, _multi_sync_section)
+            
         container_section = Gio.Menu.new()
         add_menu_action(container_section, _("Create Container Clip From Selected Clips"), "multiclipmenu.createmulticompound",  ("create_multi_compound", None), callback)
         _multi_clip_menu.append_section(None, container_section)
 
         filters_section = Gio.Menu.new()
         add_filter_menu = Gio.Menu.new()
-        _fill_filters_menus(add_filter_menu, callback, "add_filter", "multiclipmenu.addfilter.")
+        _fill_filters_menus(add_filter_menu, callback, "add_filter_multi", "multiclipmenu.addfilter.")
         filters_section.append_submenu(_("Add Filter"), add_filter_menu)
         clone_sub_menu = Gio.Menu.new()
         _fill_clone_filters_menu(clone_sub_menu, callback, True, False)
         filters_section.append_submenu(_("Clone Filter"), clone_sub_menu)
         add_menu_action(filters_section, _("Clear Filters"), "multiclipmenu.clearfilters",  ("clear_filters", None), callback)
+        add_menu_action(filters_section, _("Paste Filters"), "multiclipmenu.pastefilters",  ("paste_filters", None), callback)
         _multi_clip_menu.append_section(None, filters_section)
 
         delete_section = Gio.Menu.new()
         add_menu_action(delete_section, _("Delete"), "multiclipmenu.delete",  ("delete", None), callback)
-        add_menu_action(delete_section, _("Lift"), "multiclipmenu.delete",  ("lift", None), callback)
+        add_menu_action(delete_section, _("Lift"), "multiclipmenu.lift",  ("lift", None), callback)
+        add_menu_action(delete_section, _("Ripple Delete Selection Range"), "multiclipmenu.ripplerange",  ("ripplerangeselection", None), callback)
         _multi_clip_menu.append_section(None, delete_section)
 
+        mark_section = Gio.Menu.new()
+        add_menu_action(mark_section, _("Mark Selection Range"), "multiclipmenu.markselectionrange",  ("mark_selection_range", None), callback)
+        _multi_clip_menu.append_section(None, mark_section)
+    
     else: # Menu items with possible state changes need to recreated.
         guipopover.menu_clear_or_create(_multi_audio_section)
         _fill_multi_audio_section(_multi_audio_section, clip, track, callback)
-
+        guipopover.menu_clear_or_create(_multi_sync_section)
+        _fill_multi_sync_section(_multi_sync_section, clip, track, callback)
+        
     rect = guipopover.create_rect(x, y)
     _multi_clip_popover = guipopover.new_mouse_popover(widget, _multi_clip_menu, rect, Gtk.PositionType.TOP)
 
@@ -387,6 +412,7 @@ def transition_popover_menu_show(widget, clip, track, x, y, callback):
         _fill_clone_filters_menu(clone_sub_menu, callback, False, False, True)
         filters_section.append_submenu(_("Clone Filter"), clone_sub_menu)
         add_menu_action(filters_section, _("Clear Filters"), "transitionclipmenu.clearfilters",  ("clear_filters", None), callback)
+        add_menu_action(filters_section, _("Paste Filters"), "transitionclipmenu.pastefilters",  ("paste_filters", None), callback)
         _transition_menu.append_section(None, filters_section)
 
     else: # Menu items with possible state changes need to recreated.
@@ -401,10 +427,20 @@ def _fill_multi_audio_section(multi_audio_section, clip, track, callback):
     if clip.media_type == appconsts.IMAGE_SEQUENCE or clip.media_type == appconsts.IMAGE or clip.media_type == appconsts.PATTERN_PRODUCER:
         active = False
     add_menu_action(multi_audio_section, _("Split Audio"), "multiclipmenu.multisplitaudio",  ("multi_split_audio", None), callback, active)
-    active = (track.id == current_sequence().first_video_index)
+    active = True
     if clip.media_type == appconsts.IMAGE_SEQUENCE or clip.media_type == appconsts.IMAGE or clip.media_type == appconsts.PATTERN_PRODUCER:
         active = False
     add_menu_action(multi_audio_section, _("Split Audio Synched"), "multiclipmenu.multisplitaudiosynched",  ("multi_split_audio_synched", None), callback, active)
+    if clip.media_type == appconsts.IMAGE_SEQUENCE or clip.media_type == appconsts.IMAGE or clip.media_type == appconsts.PATTERN_PRODUCER:
+        active = False
+
+    add_menu_action(multi_audio_section, _("Mute Audio"), "multiclipmenu.muteaudio",  ("multi_mute_audio", None), callback, active)
+    add_menu_action(multi_audio_section, _("Unmute Audio"), "multiclipmenu.unmuteaudio",  ("multi_unmute_audio", None), callback, active)
+
+def _fill_multi_sync_section(sync_section, clip, track, callback):
+    add_menu_action(sync_section,_("Select Sync Parent Clip..."), "clipmenu.multisetmaster",  ("multi_set_master", None), callback)
+    add_menu_action(sync_section,_("Resync"), "clipmenu.multiresync",  ("multi_resync", None), callback)
+    add_menu_action(sync_section,_("Clear Sync Relations"), "clipmenu.multiclearsyncrel",  ("multi_clear_sync_rel", None), callback)
 
 def _fill_audio_menu(audio_submenu, clip, track, callback):
     if track.type == appconsts.VIDEO:
@@ -413,7 +449,7 @@ def _fill_audio_menu(audio_submenu, clip, track, callback):
             active = False
         add_menu_action(audio_submenu, _("Split Audio"), "clipmenu.splitaudio",  ("split_audio", None), callback, active)
 
-        active = (track.id == current_sequence().first_video_index)
+        active = True
         if clip.media_type == appconsts.IMAGE_SEQUENCE or clip.media_type == appconsts.IMAGE or clip.media_type == appconsts.PATTERN_PRODUCER:
             active = False
         add_menu_action(audio_submenu, _("Split Audio Synched"), "clipmenu.splitaudiosynched", ("split_audio_synched", None), callback, active)
@@ -494,16 +530,18 @@ def _fill_edit_actions_menu(edit_actions_menu, clip, track, callback):
     del_section = Gio.Menu.new()
     add_menu_action(del_section,_("Delete"), "clipmenu.delete",  ("delete", None), callback)
     add_menu_action(del_section,_("Lift"), "clipmenu.delete",  ("lift", None), callback)
+    add_menu_action(del_section, _("Ripple Delete Clip Range"), "clipmenu.ripplerange",  ("ripplerange", None), callback)
     edit_actions_menu.append_section(None, del_section)
 
-    if track.id != current_sequence().first_video_index:
-        sync_section = Gio.Menu.new()
-        if clip.sync_data != None:
-            add_menu_action(sync_section,_("Resync"), "clipmenu.resync",  ("resync", None), callback)
-            add_menu_action(sync_section,_("Clear Sync Relation"), "clipmenu.clearsyncrel",  ("clear_sync_rel", None), callback)
-        else:
-            add_menu_action(sync_section,_("Select Sync Parent Clip..."), "clipmenu.setmaster",  ("set_master", None), callback)
-        edit_actions_menu.append_section(None, sync_section)
+    sync_section = Gio.Menu.new()
+
+    active = (clip.sync_data != None)
+    add_menu_action(sync_section,_("Resync"), "clipmenu.resync",  ("resync", None), callback, active)
+    add_menu_action(sync_section,_("Clear Sync Relation"), "clipmenu.clearsyncrel",  ("clear_sync_rel", None), callback, active)
+
+    active = (clip.sync_data == None)
+    add_menu_action(sync_section,_("Select Sync Parent Clip..."), "clipmenu.setmaster",  ("set_master", None), callback, active)
+    edit_actions_menu.append_section(None, sync_section)
 
     length_section = Gio.Menu.new()
     add_menu_action(length_section, _("Set Clip Length..."), "clipmenu.length",  ("length", None), callback)
@@ -512,23 +550,36 @@ def _fill_edit_actions_menu(edit_actions_menu, clip, track, callback):
     edit_actions_menu.append_section(None, length_section)
 
 def _fill_tool_integration_menu(tools_sub_menu, clip, callback):
-    export_tools = toolsintegration.get_export_integrators()
-    i = 0
-    for integrator in export_tools:
-        if integrator.supports_clip_media(clip) == False:
-            active = False
-        else:
-            active = True
-        add_menu_action(tools_sub_menu, copy.copy(integrator.tool_name), "clipmenu." + str(i), (None, None),  integrator.export_callback, active)
-        i += 1
 
+    action_section = Gio.Menu.new()
+    if clip.slowmo_data != None:
+        slowmo_type, orig_file_path, slowmo_clip_media_area, speed, orig_media_in, orig_media_out = clip.slowmo_data
+    else:
+        slowmo_type = None
+    active = ((slowmo_type == None) or (slowmo_type == appconsts.SLOWMO_SLOW_FAST))
+    add_menu_action(action_section,_("Slow/Fast"), "clipslowmo.slowfast",  ("slowfast", clip), callback, active)
+    active = ((slowmo_type == None) or (slowmo_type == appconsts.SLOWMO_REVERSE))
+    add_menu_action(action_section,_("Reverse"), "clipslowmo.reverse",  ("reverse", clip), callback, active)
+    tools_sub_menu.append_section(None, action_section)
+    
+    revert_section = Gio.Menu.new()
+    active = (clip.slowmo_data != None)
+    add_menu_action(revert_section,_("Revert To Original Media"), "clipslowmo.revert",  ("revert", clip), callback, active)
+    tools_sub_menu.append_section(None, revert_section)
+    
 def _fill_select_menu(select_menu, callback, is_audio_select=False):
     if is_audio_select == True:
         pre_id = "audio"
     else:
         pre_id = ""
-    add_menu_action(select_menu, _("All Clips After"), pre_id + "clipmenu.selectallafter",  ("select_all_after", None), callback)
-    add_menu_action(select_menu, _("All Clips Before"), pre_id + "clipmenu.selectallbefore",  ("select_all_before", None), callback)
+    select_section = Gio.Menu.new()
+    add_menu_action(select_section, _("All Clips After"), pre_id + "clipmenu.selectallafter",  ("select_all_after", None), callback)
+    add_menu_action(select_section, _("All Clips Before"), pre_id + "clipmenu.selectallbefore",  ("select_all_before", None), callback)
+    select_menu.append_section(None, select_section)
+
+    mark_section = Gio.Menu.new()
+    add_menu_action(mark_section, _("Mark Clip Range"), pre_id + "clipmenu.markcliprange",  ("mark_clip_range", None), callback)
+    select_menu.append_section(None, mark_section)
 
 def _fill_filters_menus(sub_menu, callback, item_id, action_id):
     j = 0
@@ -570,20 +621,33 @@ def _fill_compositors_section(compositor_section, clip, track, callback):
     add_menu_action(compositor_section, _("Delete Compositor"), "clipmenu.deletecompositors",  ("delete_compositors", None), callback, active)
 
 def _fill_title_section(title_section, clip, callback):
-    #if current_sequence().compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
     active = (clip.titler_data != None)
     add_menu_action(title_section, _("Edit Title"), "clipmenu.edittitle",  ("edit_title", None), callback, active)
-    
+
 def _fill_generator_section(generator_section, clip, callback):
+        
     active = (clip.container_data != None)
     genactive = active
     if clip.container_data != None and clip.container_data.container_type != appconsts.CONTAINER_CLIP_FLUXITY:
         genactive = False
+    seq_link_active = False 
+    if clip.container_data != None and clip.container_data.container_type == appconsts.CONTAINER_CLIP_SEQUENCE_LINK:
+        seq_link_active = True
+
     add_menu_action(generator_section, _("Edit Generator Properties..."), "clipmenu.ccedit",  ("cc_edit_program", None), callback, genactive)
     add_menu_action(generator_section, _("Create Cloned Generator..."), "clipmenu.ccclonegen",  ("cc_clone_generator", None), callback, genactive)
-    add_menu_action(generator_section, _("Generator/Container Render Actions"), "clipmenu.ccrender",  ("cc_render_clip", None), callback, active)
-    add_menu_action(generator_section, _("Switch to Unrendered Media"), "clipmenu.ccgotounrendered", ("cc_go_to_underdered", None),  callback, active)
+    add_menu_action(generator_section, _("Update Sequence Link Clip"), "clipmenu.updateseqlink", ("cc_update_seq_link", None),  callback, seq_link_active)
 
+    generator_section.append_submenu(_("Generator and Container Rendering"), _generator_sub_menu)
+        
+def _fill_generator_render_section(generator_section, clip, callback):
+    active = (clip.container_data != None)
+    
+    add_menu_action(generator_section, _("Render Clip..."), "clipmenu.ccrender",  ("cc_render_clip", None), callback, active)
+    add_menu_action(generator_section, _("Render Settings..."), "clipmenu.ccencoding",  ("cc_render_settings", None), callback, active)
+    add_menu_action(generator_section, _("Switch to Unrendered Media"), "clipmenu.ccgotounrendered", ("cc_go_to_underdered", None),  callback, active)
+    add_menu_action(generator_section, _("Set Default Encoding..."), "clipmenu.ccdefaultencoding", ("cc_default_encoding", None),  callback, active)
+    
 def _fill_audio_clip_sync_section(sync_section, clip, callback):
     is_synched = (clip.sync_data != None)
     add_menu_action(sync_section, _("Resync"), "audioclipmenu.resync", ("resync", None), callback, is_synched)
@@ -605,6 +669,7 @@ def _fill_audio_edit_actions_menu(edit_actions_menu, callback):
     del_section = Gio.Menu.new()
     add_menu_action(del_section,_("Delete"), "audioclipmenu.delete",  ("delete", None), callback)
     add_menu_action(del_section,_("Lift"), "audioclipmenu.delete",  ("lift", None), callback)
+    add_menu_action(del_section, _("Ripple Delete Clip Range"), "audioclipmenu.ripplerange",  ("ripplerange", None), callback)
     edit_actions_menu.append_section(None, del_section)
 
     length_section = Gio.Menu.new()
@@ -663,6 +728,7 @@ def kftool_popover_menu_show(widget, kftool, x, y, callback, callback_snapping):
         _kftool_menu.append_section(None, plyhead_section)
 
         exit_section = Gio.Menu.new()
+        add_menu_action(exit_section, _("Open in Edit Panel Keyframe Editor"), "kftoolmenu.editpanel",  ("editpanel", None), callback)
         add_menu_action(exit_section, _("Exit Edit"), "kftoolmenu.exit",  ("exit", None), callback)
         _kftool_menu.append_section(None, exit_section)
 
@@ -740,7 +806,7 @@ def kftype_select_popover_menu_show(widget, kf_type, x, y, callback):
     global _kf_select_popover, _kf_select_menu
     
     _kf_select_menu = guipopover.menu_clear_or_create(_kf_select_menu)
-    
+    """
     items_data = [( _("Linear"), "linear"), ( _("Smooth"), "smooth"), ( _("Discrete"), "discrete")]
     if kf_type == appconsts.KEYFRAME_LINEAR:
         active_index = 0
@@ -749,15 +815,42 @@ def kftype_select_popover_menu_show(widget, kf_type, x, y, callback):
     else:
         active_index = 2
 
-    guipopover.add_menu_action_all_items_radio(_kf_select_menu, items_data, "kftoolkftypemenu.selecttype", active_index, callback)
-        
+    kftype_section = Gio.Menu.new()    
+    guipopover.add_menu_action_all_items_radio(kftype_section, items_data, "kftoolkftypemenu.selecttype", active_index, callback)
+    _kf_select_menu.append_section(None, kftype_section)
     """
+    
+    text_smooth = ""
+    text_effect = ""
+    if kf_type == appconsts.KEYFRAME_LINEAR:
+        active_index = 0
+    elif kf_type == appconsts.KEYFRAME_SMOOTH:
+        active_index = 1
+    elif kf_type == appconsts.KEYFRAME_DISCRETE:
+        active_index = 2
+    elif kf_type in animatedvalue.SMOOTH_EXTENDED_KEYFRAME_TYPES:
+        active_index = 3
+        text_smooth = animatedvalue.TYPE_TO_NAME[kf_type] +  " - "
     else:
-        guipopover.menu_clear_or_create(_params_submenu)
-        
-        guipopover.add_menu_action_all_items_radio(menu, items_data, action_id, active_index, callback)
-    """
+        active_index = 4
+        text_effect = animatedvalue.TYPE_TO_NAME[kf_type] +  " - "
 
+    items_data = [( _("Linear"), str(appconsts.KEYFRAME_LINEAR)), 
+                  ( _("Smooth"), str(appconsts.KEYFRAME_SMOOTH)), 
+                  ( _("Discrete"), str(appconsts.KEYFRAME_DISCRETE)), 
+                  ( text_smooth + _("Smooth Extended") + "...", "smoothkfs"),
+                  ( text_effect + _("Effect") + "...", "effectkfs")]
+                  
+    kftype_section = Gio.Menu.new()   
+    guipopover.add_menu_action_all_items_radio(kftype_section, items_data, "kftoolkftypemenu.selecttype", active_index, callback)
+    _kf_select_menu.append_section(None, kftype_section)
+
+    kfcopy_section = Gio.Menu.new()
+    add_menu_action(kfcopy_section,_("Copy Keyframe Value"), "kftoolkftypemenu.copykf",  ("copykf", None), callback)
+    add_menu_action(kfcopy_section,_("Paste Keyframe Value"), "kftoolkftypemenu.pastekf",  ("pastekf", None), callback)
+    add_menu_action(kfcopy_section,_("Copy Keyframe Value From Next"), "kftoolkftypemenu.clonenextkf",  ("clonekfnext", None), callback)
+    add_menu_action(kfcopy_section,_("Copy Keyframe Value From Previous"), "kftoolkftypemenu.cloneprevkf",  ("clonekfprev", None), callback)
+    _kf_select_menu.append_section(None, kfcopy_section)
 
     rect = guipopover.create_rect(x, y)
     _kf_select_popover = guipopover.new_mouse_popover(widget, _kf_select_menu, rect, Gtk.PositionType.TOP)
