@@ -29,7 +29,6 @@ from gi.repository import Gtk, Gdk, GLib
 from gi.repository import GdkPixbuf
 
 import appconsts
-import editorpersistance
 import respaths
 import translations
 
@@ -98,6 +97,17 @@ def get_two_column_box(widget1, widget2, left_width):
     hbox.pack_start(widget2, True, True, 0)
     return hbox
 
+def get_two_column_box_fixed(widget1, widget2, left_width, right_width, margin):
+    hbox = Gtk.HBox()
+    left_box = get_left_justified_box([widget1])
+    left_box.set_size_request(left_width, TWO_COLUMN_BOX_HEIGHT)
+    right_box = get_left_justified_box([widget2])
+    right_box.set_size_request(right_width, TWO_COLUMN_BOX_HEIGHT)
+    right_box.set_margin_start(margin)
+    hbox.pack_start(left_box, False, False, 0)
+    hbox.pack_start(right_box, False, False, 0)
+    return hbox
+    
 def get_three_column_box(widget1, widget2, widget3, left_width, right_width):
     hbox = Gtk.HBox()
     left_box = get_left_justified_box([widget1])
@@ -148,7 +158,6 @@ def get_right_expand_box(widget1, widget2, center_pad=False):
     hbox.pack_start(widget2, True, True, 0)
     return hbox
 
-# Aug-2019 - SvdB - BB
 def get_image_name(img_name, suffix = ".png", double_height = False):
     button_size_text = ""
     if double_height:
@@ -156,78 +165,24 @@ def get_image_name(img_name, suffix = ".png", double_height = False):
     img_name = img_name+button_size_text+suffix
     return img_name
 
-# Aug-2019 - SvdB - BB
 def get_image(img_name, suffix = ".png", force = None):
-    # Use parameter force as True or False to force the track height no matter what the preferences setting
-    if force == None:
-        force = editorpersistance.prefs.double_track_hights
-    if force:
-        new_name = img_name + "@2"
-    else:
-        new_name = img_name
-    try:
-        img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + new_name + suffix)
-    except:
-        img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + img_name + suffix)
+    img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + img_name + suffix)
     return img
-    
 
-# Aug-2019 - SvdB - BB
 def get_cairo_image(img_name, suffix = ".png", force = None):
-    # Apr-2020 - SvdB - Make it sturdier in case a @2 image is missing. Just display the original image.
-    # Use parameter force as True or False to force the track height no matter what the preferences setting
-    if force == None:
-        force = editorpersistance.prefs.double_track_hights
-    if force:
-        new_name = img_name + "@2"
-    else:
-        new_name = img_name
-
-    try:
-        img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + new_name + suffix)
-    except:
-        # Colorized icons
-        if img_name[-6:] == "_color":  #editorpersistance.prefs.colorized_icons is True:
-            try:
-                img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + "_color" + suffix)
-            except:
-                img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name[:-6] + suffix)
-        else:
-            img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + suffix)
-        # End of Colorized icons
+    img = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + img_name + suffix)
     return img
 
-def get_double_scaled_cairo_image(icon_name):
-    img_path = respaths.IMAGE_PATH + icon_name
-    
-    icon = cairo.ImageSurface.create_from_png(img_path)
-    if editorpersistance.prefs.double_track_hights == False:
-        return icon
-    
-    surface_pattern = cairo.SurfacePattern(icon)
-    surface_pattern.set_filter(cairo.Filter.NEAREST)
-    
-    scaled_icon = cairo.ImageSurface(cairo.FORMAT_ARGB32, icon.get_width() * 2, icon.get_height() * 2)
-    cr = cairo.Context(scaled_icon)
-    cr.scale(2.0, 2.0)
-    cr.set_source(surface_pattern)
-    cr.paint()
-
-    return scaled_icon
-        
-# Aug-2019 - SvdB - BB
 def get_image_button(img_file_name, width, height):
     button = Gtk.Button()
     icon = get_image(img_file_name)        
     size_adj = 1
-    if editorpersistance.prefs.double_track_hights:
-        size_adj = 2
     button_box = Gtk.HBox()
     button_box.pack_start(icon, False, False, 0)
     button.add(button_box)
-    button.set_size_request(width*size_adj, height*size_adj)
+    button.set_size_request(width, height)
     return button
-    
+
 def get_pad_label(w, h):
     label = Gtk.Label()
     label.set_size_request(w, h)
@@ -420,25 +375,6 @@ def get_menu_item(text, callback, data, sensitive=True):
     item.set_sensitive(sensitive)
     return item
 
-
-def get_image_menu_item(text, image_name, callback, tooltip_markup=None):
-    img = get_image(image_name)
-    text_label = Gtk.Label(label=text)
-    
-    hbox = Gtk.HBox()
-    hbox.pack_start(pad_label(4, 4), False, False, 0)
-    hbox.pack_start(img, False, False, 0)
-    hbox.pack_start(pad_label(4, 4), False, False, 0)
-    hbox.pack_start(text_label, False, False, 0)
-    hbox.show_all()
-    item = Gtk.MenuItem()
-    item.add(hbox)
-    if tooltip_markup != None:
-        item.set_tooltip_markup(tooltip_markup)
-    item.show()
-
-    return item
-    
 def get_radio_menu_items_group(menu, labels, msgs, callback, active_index):
     first_item = Gtk.RadioMenuItem()
     first_item.set_label(labels[0])
