@@ -18,19 +18,19 @@
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
+import multiprocessing
 import os
 
 from gi.repository import Gtk
 
 import appconsts
+import databridge
 import dialogutils
 import editorpersistance
 import gui
 import guiutils
+import gtkbuilder
 import mltprofiles
-import multiprocessing
-import usbhid
-import utils
 import utilsgtk
 
 
@@ -41,7 +41,7 @@ PREFERENCES_LEFT = 410
 def preferences_dialog():
 
     dialog = Gtk.Dialog(_("Editor Preferences"), None,
-                    Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                    None,
                     (_("Cancel"), Gtk.ResponseType.REJECT,
                     _("OK"), Gtk.ResponseType.ACCEPT))
 
@@ -123,15 +123,15 @@ def _general_options_panel():
     load_order_combo.append_text(_("Absolute paths only"))
     load_order_combo.set_active(prefs.media_load_order)
 
-    render_folder_select = Gtk.FileChooserButton.new (_("Select Default Render Folder"), Gtk.FileChooserAction.SELECT_FOLDER)
+    render_folder_select = gtkbuilder.get_file_chooser_button(_("Select Default Render Folder"), Gtk.FileChooserAction.SELECT_FOLDER)
     if prefs.default_render_directory == None or prefs.default_render_directory == appconsts.USER_HOME_DIR \
         or (not os.path.exists(prefs.default_render_directory)) \
         or (not os.path.isdir(prefs.default_render_directory)):
-        render_folder_select.set_current_folder_uri(os.path.expanduser("~") + "/")
+        render_folder_select.set_current_folder(os.path.expanduser("~") + "/")
     else:
-        render_folder_select.set_current_folder_uri(prefs.default_render_directory)
+        render_folder_select.set_current_folder(prefs.default_render_directory)
 
-    disk_cache_warning_combo  = Gtk.ComboBoxText()
+    disk_cache_warning_combo = Gtk.ComboBoxText()
     disk_cache_warning_combo.append_text(_("Off"))
     disk_cache_warning_combo.append_text(_("500 MB"))
     disk_cache_warning_combo.append_text(_("1 GB"))
@@ -250,23 +250,12 @@ def _playback_prefs_panel():
     auto_center_on_stop = Gtk.CheckButton()
     auto_center_on_stop.set_active(prefs.auto_center_on_play_stop)
 
-    # Jul-2016 - SvdB - For play_pause button
-    play_pause_button = Gtk.CheckButton()
-    # The following test is to make sure play_pause can be used for the initial value. If not found, then leave uninitialized
-    if hasattr(prefs, 'play_pause'):
-        play_pause_button.set_active(prefs.play_pause)
-
-    timeline_start_end_button = Gtk.CheckButton()
-    if hasattr(prefs, 'timeline_start_end'):
-        timeline_start_end_button.set_active(prefs.timeline_start_end)
-
     auto_center_on_updown = Gtk.CheckButton()
     auto_center_on_updown.set_active(prefs.center_on_arrow_move)
 
     follow_move_range = Gtk.CheckButton()
     follow_move_range.set_active(prefs.playback_follow_move_tline_range)
 
-    # Apr-2017 - SvdB - For FF/Rev speed options
     if hasattr(prefs, 'ffwd_rev_shift'):
         spin_adj = Gtk.Adjustment(value=prefs.ffwd_rev_shift, lower=1, upper=10, step_increment=1)
     else:
@@ -288,8 +277,6 @@ def _playback_prefs_panel():
 
     row2 = _row(guiutils.get_checkbox_row_box(auto_center_on_stop, Gtk.Label(label=_("Center Current Frame on Playback Stop"))))
     row13 = _row(guiutils.get_checkbox_row_box(auto_center_on_updown, Gtk.Label(label=_("Center Current Frame after Up/Down Arrow"))))
-    row10 = _row(guiutils.get_checkbox_row_box(play_pause_button, Gtk.Label(label=_("Enable single Play/Pause button"))))
-    row11 = _row(guiutils.get_checkbox_row_box(timeline_start_end_button, Gtk.Label(label=_("Enable To Start and To End buttons"))))
     row14 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Fast Forward / Reverse Speed for Shift Key:")), ffwd_rev_shift_spin, PREFERENCES_LEFT))
     row14.set_tooltip_text(_("Speed of Forward / Reverse will be multiplied by this value if Shift Key is held (Only using KEYS).\n" \
         "Enabling multiple modifier keys will multiply the set values.\n" \
@@ -306,8 +293,6 @@ def _playback_prefs_panel():
     vbox.pack_start(row18, False, False, 0)
     vbox.pack_start(row2, False, False, 0)
     vbox.pack_start(row13, False, False, 0)
-    vbox.pack_start(row10, False, False, 0)
-    vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(row14, False, False, 0)
     vbox.pack_start(row16, False, False, 0)
 
@@ -315,9 +300,8 @@ def _playback_prefs_panel():
 
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    return vbox, (auto_center_on_stop,
-                  play_pause_button, timeline_start_end_button, auto_center_on_updown,
-                  ffwd_rev_shift_spin, ffwd_rev_caps_spin, follow_move_range, loop_clips)
+    return vbox, (auto_center_on_stop, auto_center_on_updown, ffwd_rev_shift_spin,
+                  ffwd_rev_caps_spin, follow_move_range, loop_clips)
 
 def _view_prefs_panel():
     prefs = editorpersistance.prefs
@@ -346,10 +330,6 @@ def _view_prefs_panel():
     force_language_combo.set_active(active_index)
     force_language_combo.lang_codes = lang_list
 
-    display_splash_check = Gtk.CheckButton()
-    display_splash_check.set_active(prefs.display_splash_screen)
-
-    # Feb-2017 - SvdB - For full file names
     show_full_file_names = Gtk.CheckButton()
     show_full_file_names.set_active(prefs.show_full_file_names)
 
@@ -362,16 +342,10 @@ def _view_prefs_panel():
         window_mode_combo.set_active(1)
 
     tracks_combo = Gtk.ComboBoxText()
-    tracks_combo.append_text(_("Normal - 50px, 25px"))
-    tracks_combo.append_text(_("Double for HiDPI - 100px, 50px"))
-    # Aug-2019 - SvdB - BB
-    tracks_combo.set_active(prefs.double_track_hights)
-
-    # DEPRECATED, remove.
-    top_row_layout = Gtk.ComboBoxText()
-    top_row_layout.append_text(_("3 panels if width (1450px+) available"))
-    top_row_layout.append_text(_("2 panels always"))
-    top_row_layout.set_active(prefs.top_row_layout)
+    tracks_combo.append_text(_("Default - 50px, 25px"))
+    tracks_combo.append_text(_("1.5 x - 75px, 37px"))
+    tracks_combo.append_text(_("2 x - 100px, 50px"))
+    tracks_combo.set_active(prefs.tracks_scale)
 
     monitors_data = utilsgtk.get_display_monitors_size_data()
     layout_monitor = Gtk.ComboBoxText()
@@ -389,31 +363,50 @@ def _view_prefs_panel():
     filter_select_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.FILTER_SELECT_WIDTH_MIN, editorpersistance.FILTER_SELECT_WIDTH_MAX, 1)
     filter_select_width_spin.set_adjustment(spin_adj)
     filter_select_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.project_panel_width, lower=editorpersistance.PROJECT_PANEL_WIDTH_MIN, upper=editorpersistance.PROJECT_PANEL_WIDTH_MAX, step_increment=1)
+    project_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.PROJECT_PANEL_WIDTH_MIN, editorpersistance.PROJECT_PANEL_WIDTH_MAX, 1)
+    project_panel_width_spin.set_adjustment(spin_adj)
+    project_panel_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.editor_panel_width, lower=editorpersistance.EDIT_PANEL_WIDTH_MIN, upper=editorpersistance.EDIT_PANEL_WIDTH_MAX, step_increment=1)
+    edit_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.EDIT_PANEL_WIDTH_MIN, editorpersistance.EDIT_PANEL_WIDTH_MAX, 1)
+    edit_panel_width_spin.set_adjustment(spin_adj)
+    edit_panel_width_spin.set_numeric(True)
+
+    spin_adj = Gtk.Adjustment(value=prefs.media_panel_width, lower=editorpersistance.MEDIA_PANEL_WIDTH_MIN, upper=editorpersistance.MEDIA_PANEL_WIDTH_MAX, step_increment=1)
+    media_panel_width_spin = Gtk.SpinButton.new_with_range(editorpersistance.MEDIA_PANEL_WIDTH_MIN, editorpersistance.MEDIA_PANEL_WIDTH_MAX, 1)
+    media_panel_width_spin.set_adjustment(spin_adj)
+    media_panel_width_spin.set_numeric(True)
     
     row00 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Application window mode:")), window_mode_combo, PREFERENCES_LEFT))
     row9 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Force Language:")), force_language_combo, PREFERENCES_LEFT))
-    row1 = _row(guiutils.get_checkbox_row_box(display_splash_check, Gtk.Label(label=_("Display splash screen"))))
+    row6 = _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
     row7 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Tracks Heights:")), tracks_combo, PREFERENCES_LEFT))
-    # Feb-2017 - SvdB - For full file names
-    row6 =  _row(guiutils.get_checkbox_row_box(show_full_file_names, Gtk.Label(label=_("Show Full File names"))))
+
     row10 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Do GUI layout based on:")), layout_monitor, PREFERENCES_LEFT))
+    row13 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Edit panel width:")), edit_panel_width_spin, PREFERENCES_LEFT))
+    row12 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Project panel width:")), project_panel_width_spin, PREFERENCES_LEFT))
     row11 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Filter Select panel width:")), filter_select_width_spin, PREFERENCES_LEFT))
+    row14 = _row(guiutils.get_two_column_box(Gtk.Label(label=_("Media panel width:")), media_panel_width_spin, PREFERENCES_LEFT))
     
     vbox = Gtk.VBox(False, 2)
     vbox.pack_start(row00, False, False, 0)
     vbox.pack_start(row10, False, False, 0)
     vbox.pack_start(row9, False, False, 0)
-    vbox.pack_start(row1, False, False, 0)
-    vbox.pack_start(row7, False, False, 0)
-    # Feb-2017 - SvdB - For full file names
     vbox.pack_start(row6, False, False, 0)
+    vbox.pack_start(row7, False, False, 0)
+    vbox.pack_start(row14, False, False, 0)
+    vbox.pack_start(row13, False, False, 0)
+    vbox.pack_start(row12, False, False, 0)
     vbox.pack_start(row11, False, False, 0)
     vbox.pack_start(Gtk.Label(), True, True, 0)
     
     guiutils.set_margins(vbox, 12, 0, 12, 12)
 
-    return vbox, (force_language_combo, display_splash_check, window_mode_combo, show_full_file_names,
-                  tracks_combo, top_row_layout, layout_monitor, filter_select_width_spin)
+    return vbox, (force_language_combo, window_mode_combo, show_full_file_names,
+                  tracks_combo, project_panel_width_spin, edit_panel_width_spin, media_panel_width_spin,
+                  layout_monitor, filter_select_width_spin)
 
 
 
@@ -477,7 +470,7 @@ def _jog_shuttle_panel():
 
     usbhid_config_combo = Gtk.ComboBoxText()
     usbhid_config_combo.append_text("None")
-    for usbhid_config_metadata in usbhid.get_usb_hid_device_config_metadata_list():
+    for usbhid_config_metadata in databridge.usbhid_get_usb_hid_device_config_metadata_list():
         usbhid_config_combo_index += 1
         usbhid_config_combo.append_text(usbhid_config_metadata.name)
 
