@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2014 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,17 +24,13 @@ import gi
 gi.require_version('Gtk', '3.0')
 
 from gi.repository import GObject, GLib, Gio
-from gi.repository import Gtk, Gdk, GdkPixbuf
+from gi.repository import Gtk, GdkPixbuf
 
-import dbus
-import dbus.service
-from dbus.mainloop.glib import DBusGMainLoop
 try:
     import mlt7 as mlt
 except:
     import mlt
 import hashlib
-import locale
 import os
 from os import listdir
 from os.path import isfile, join
@@ -55,13 +51,13 @@ import editorstate
 import editorpersistance
 import gui
 import guiutils
+import gtkbuilder
 import mltinit
 import mltprofiles
-import processutils
 import persistance
 import respaths
 import renderconsumer
-import translations
+import toolguicomponents
 import userfolders
 import utils
 
@@ -94,15 +90,18 @@ queue_runner_thread = None
 
 timeout_id = None
 
-_dbus_service = None
+_ipc_handle = None
+APP_LOCK_FILE = "batch_render_app_lock"
 
-render_item_menu = Gtk.Menu()
+_render_item_popover = None
+_render_item_menu = None
 
 _single_render_app = None
 single_render_window = None
 single_render_launch_thread = None
 single_render_thread = None
 
+
 # -------------------------------------------------------- render thread
 class QueueRunnerThread(threading.Thread):
     def __init__(self):
@@ -202,7 +201,7 @@ class QueueRunnerThread(threading.Thread):
 
     def _queue_done_update(self):
         # Update view for render end
-        batch_window.reload_queue() # item may havee added to queue while rendering
+        batch_window.reload_queue() # item may have added to queue while rendering
         batch_window.render_queue_stopped()
         
     def abort(self):
@@ -212,27 +211,74 @@ class QueueRunnerThread(threading.Thread):
         self.running = False
         self.thread_running = False
         
-        batch_window.reload_queue() # item may havee added to queue while rendering
+        batch_window.reload_queue() # item may have added to queue while rendering
 
 
-class BatchRenderDBUSService(dbus.service.Object):
+class BatchRenderIPC():
     def __init__(self):
-        print("dbus service init")
-        bus_name = dbus.service.BusName('flowblade.movie.editor.batchrender', bus=dbus.SessionBus())
-        dbus.service.Object.__init__(self, bus_name, '/flowblade/movie/editor/batchrender')
+        self.polling_thread = None
 
-    @dbus.service.method('flowblade.movie.editor.batchrender')
-    def render_item_added(self):
-        if queue_runner_thread == None:
-            batch_window.reload_queue()
+    def app_running(self):
+        if isfile(self.get_lockfile()) == False:
+            return False
         
-        return "OK"
+        # We may just try to read lock file while it is just being written to.
+        timestamp = None
+        attempts = 0
+        while attempts < 10 and timestamp == None:
+            try:
+                #f = open(self.get_lockfile())
+                timestamp = float(utils.unpickle(self.get_lockfile()))
+                #f.close()
+            except:
+                pass
+            attempts += 1
+    
+        # Should not happen.
+        if timestamp == None:
+            print("Failed to read timestamp in BatchRenderIPC.app_running()")
+            return False
 
-    @dbus.service.method('flowblade.movie.editor.batchrender')
-    def remove_from_dbus(self):
-        self.remove_from_connection()
-        return
-   
+        # Running batch render app lockfile was timestamp longer then 3 ago or
+        # in the future we assume that we don't have reliable info on running app.
+        if time.time() - timestamp > 3.0 or timestamp > time.time() :
+            # We assume that 
+            os.remove(self.get_lockfile())
+            return False
+        
+        return True
+
+    def launch_polling(self, callback_func):
+        self.polling_thread = StatusPollingThread(callback_func)
+        self.polling_thread.start()
+
+    def stop_polling(self):
+        if self.polling_thread != None:
+            self.polling_thread.running = False
+
+    def write_running_timestamp(self):
+        with atomicfile.AtomicFileWriter(self.get_lockfile(), "wb") as afw:
+            write_file = afw.get_file()
+            pickle.dump(str(time.time()), write_file)
+    
+    def delete_running_timestamp(self):
+        os.remove(self.get_lockfile())
+
+    def get_lockfile(self):
+        return userfolders.get_cache_dir() + APP_LOCK_FILE
+
+
+class StatusPollingThread(threading.Thread):
+    def __init__(self, polling_callback):
+        threading.Thread.__init__(self)
+        self.polling_callback = polling_callback
+    
+    def run(self):        
+        self.running = True
+        while self.running:
+            self.polling_callback()
+            time.sleep(2.0)
+    
 
 # --------------------------------------------------- adding item, always called from main app
 def add_render_item(flowblade_project, render_path, args_vals_list, mark_in, mark_out, render_data):
@@ -260,13 +306,9 @@ def add_render_item(flowblade_project, render_path, args_vals_list, mark_in, mar
     # Write render item file
     render_item.save()
 
-    bus = dbus.SessionBus()
-    if bus.name_has_owner('flowblade.movie.editor.batchrender'):
-        obj = bus.get_object('flowblade.movie.editor.batchrender', '/flowblade/movie/editor/batchrender')
-        iface = dbus.Interface(obj, 'flowblade.movie.editor.batchrender')
-        iface.render_item_added()
-    else:
-        launch_batch_rendering()
+    # Launch app if not already running.
+    launch_batch_rendering()
+
 
 # ------------------------------------------------------- file utils
 def init_dirs_if_needed():
@@ -323,14 +365,16 @@ def maybe_create_render_folder(render_path):
         
 # --------------------------------------------------------------- app thread and data objects
 def launch_batch_rendering():
-    bus = dbus.SessionBus()
-    if bus.name_has_owner('flowblade.movie.editor.batchrender'):
-        print("flowblade.movie.editor.batchrender dbus service exists, batch rendering already running")
+    ipc_handle = BatchRenderIPC()
+    if ipc_handle.app_running() == True:
         _show_single_instance_info()
     else:
-        FLOG = open(userfolders.get_cache_dir() + "log_batch_render", 'w')
-        subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowbladebatch"], stdin=FLOG, stdout=FLOG, stderr=FLOG)
+        _do_batch_render_launch()
 
+def _do_batch_render_launch():
+    FLOG = open(userfolders.get_cache_dir() + "log_batch_render", 'w')
+    subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowbladebatch"], stdin=FLOG, stdout=FLOG, stderr=FLOG)
+        
 def main(root_path, force_launch=False):
     gtk_version = "%s.%s.%s" % (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())
     editorstate.gtk_version = gtk_version
@@ -362,7 +406,7 @@ class BatchRenderApp(Gtk.Application):
         self.connect("activate", self.on_activate)
 
     def on_activate(self, data=None):
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
 
         mltinit.init_with_translations()
         
@@ -378,9 +422,9 @@ class BatchRenderApp(Gtk.Application):
             secondary_txt = _("Message:\n") + render_queue.get_error_status_message()
             dialogutils.warning_message(primary_txt, secondary_txt, batch_window.window)
 
-        DBusGMainLoop(set_as_default=True)
-        global _dbus_service
-        _dbus_service = BatchRenderDBUSService()
+        global _ipc_handle
+        _ipc_handle = BatchRenderIPC()
+        _ipc_handle.launch_polling(batch_window.poll_status)
 
         self.add_window(batch_window.window)
         
@@ -392,7 +436,7 @@ def _display_single_instance_window():
     GObject.source_remove(timeout_id)
     primary_txt = _("Batch Render Queue already running!")
 
-    msg = _("Batch Render Queue application was detected in session dbus.")
+    msg = _("Batch Render Queue application was detected running.")
 
     content = dialogutils.get_warning_message_dialog_panel(primary_txt, msg, True)
     align = dialogutils.get_default_alignment(content)
@@ -418,12 +462,11 @@ def shutdown():
         dialogutils.info_message(primary_txt, secondary_txt, batch_window.window)
         return True # Tell callsite (inside GTK toolkit) that event is handled, otherwise it'll destroy window anyway.
 
-    # GTK4 
-    while(Gtk.events_pending()):
-        Gtk.main_iteration()
+    while(GLib.MainContext.default ().pending()):
+        GLib.MainContext.default().iteration(False) # GLib.MainContext to replace this
 
-    if _dbus_service != None:
-        _dbus_service.remove_from_dbus()
+    _ipc_handle.stop_polling()
+    _ipc_handle.delete_running_timestamp()
 
     _batch_render_app.quit()
 
@@ -458,8 +501,6 @@ class RenderQueue:
                 if self.error_status == None:
                     self.error_status = []
                 self.error_status.append((data_file_name, _(" project file load failed with ") + str(e)))
-
-        print(self.error_status)
          
         if self.error_status != None:
             for file_path, error_str in self.error_status:
@@ -505,7 +546,18 @@ class RenderQueue:
         
         return same_paths
 
+    def queue_has_hanged(self, test_queue):
+        if len(self.queue) != len(test_queue.queue):
+            return True
         
+        for i in range(0, len(self.queue)):
+            if self.queue[i].generate_identifier() != test_queue.queue[i].generate_identifier():
+                return True
+        
+        return False
+
+
+
 class BatchRenderItemData:
     def __init__(self, project_name, sequence_name, render_path, sequence_index, \
                  args_vals_list, timestamp, length, mark_in, mark_out, render_data):
@@ -769,6 +821,15 @@ class BatchRenderWindow:
         
         dialog.destroy()
 
+    def poll_status(self):
+        _ipc_handle.write_running_timestamp()
+        
+        test_queue = RenderQueue()
+        test_queue.load_render_items()
+        
+        if render_queue.queue_has_hanged(test_queue) == True:
+            self.reload_queue()
+
     def reload_queue(self):
         global render_queue
         render_queue = RenderQueue()
@@ -867,7 +928,6 @@ class RenderQueueView(Gtk.VBox):
         # Scroll container
         self.scroll = Gtk.ScrolledWindow()
         self.scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
-        self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
         self.treeview = Gtk.TreeView(model=self.storemodel)
@@ -959,12 +1019,12 @@ class RenderQueueView(Gtk.VBox):
                 path, col, cellx, celly = pthinfo
                 treeview.grab_focus()
                 treeview.set_cursor(path, col, 0)
-                display_render_item_popup_menu(self.item_menu_item_selected, event)
+                render_item_popover_show(treeview, event.x, event.y, self.item_menu_item_selected)
             return True
         else:
             return False
 
-    def item_menu_item_selected(self, widget, msg):
+    def item_menu_item_selected(self, action, variant, msg):
         model, rows = self.treeview.get_selection().get_selected_rows()
         render_item = render_queue.queue[max(rows[0])]
         if msg == "renderinfo":
@@ -1070,8 +1130,8 @@ def show_change_render_item_path_dialog(callback, render_item):
     cancel_str = _("Cancel")
     ok_str = _("Change Path")
     dialog = Gtk.Dialog(_("Change Render Item Path"),
-                        batch_window.window,
-                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                        None,
+                        None,
                         (cancel_str, Gtk.ResponseType.CANCEL,
                         ok_str, Gtk.ResponseType.YES))
 
@@ -1079,7 +1139,7 @@ def show_change_render_item_path_dialog(callback, render_item):
     
     folder, f_name = os.path.split(render_item.render_path)
 
-    out_folder = Gtk.FileChooserButton(_("Select target folder"))
+    out_folder = gtkbuilder.get_file_chooser_button(_("Select target folder"))
     out_folder.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
     out_folder.set_current_folder(folder)
     
@@ -1120,29 +1180,26 @@ def _change_render_item_path_callback(dialog, response_id, data):
     else:
         dialog.destroy()
         
-def display_render_item_popup_menu(callback, event):
-    menu = render_item_menu
-    guiutils.remove_children(menu)
-    
-    menu.add(_get_menu_item(_("Change Item Render File Path..."), callback,"changepath"))
-    menu.add(_get_menu_item(_("Save Item Project As..."), callback,"saveas"))
-    menu.add(_get_menu_item(_("Render Properties"), callback,"renderinfo")) 
-    _add_separetor(menu)
-    menu.add(_get_menu_item(_("Delete"), callback,"delete"))
-    menu.popup(None, None, None, None, event.button, event.time)
-    
-def _add_separetor(menu):
-    sep = Gtk.SeparatorMenuItem()
-    sep.show()
-    menu.add(sep)
+def render_item_popover_show(widget, x, y, callback):
 
-def _get_menu_item(text, callback, data, sensitive=True):
-    item = Gtk.MenuItem(text)
-    item.connect("activate", callback, data)
-    item.show()
-    item.set_sensitive(sensitive)
-    return item
+    global _render_item_popover, _render_item_menu
+    _render_item_menu = toolguicomponents.menu_clear_or_create(_render_item_menu)
 
+    main_section = Gio.Menu.new()
+    toolguicomponents.add_menu_action(_batch_render_app, main_section, _("Change Item Render File Path..."), "renderitem.changepath", "changepath", callback)
+    toolguicomponents.add_menu_action(_batch_render_app, main_section, _("Save Item Project As..."), "renderitem.saveas", "saveas", callback)
+    toolguicomponents.add_menu_action(_batch_render_app, main_section, _("Render Properties"), "renderitem.renderinfo", "renderinfo", callback)
+    _render_item_menu.append_section(None, main_section)
+
+    delete_section = Gio.Menu.new()    
+    toolguicomponents.add_menu_action(_batch_render_app, delete_section,_("Delete"), "renderitem.deleta", "delete", callback)
+    _render_item_menu.append_section(None, delete_section)
+
+    rect = toolguicomponents.create_rect(x - 1, y + 24)
+    
+    _render_item_popover = Gtk.Popover.new_from_model(widget, _render_item_menu)
+    _render_item_popover.set_pointing_to(rect) 
+    _render_item_popover.show()
 
 
 # --------------------------------------------------- single item render
@@ -1220,7 +1277,7 @@ class SingleRenderApp(Gtk.Application):
         self.connect("activate", self.on_activate)
 
     def on_activate(self, data=None):
-        gui.apply_theme(editorpersistance.prefs.theme)
+        gui.apply_theme()
 
         mltinit.init_with_translations()
     
@@ -1270,7 +1327,7 @@ class SingleRenderThread(threading.Thread):
         
         # We just autocreate folder if for some reason it has been deleted.
         maybe_create_render_folder(render_item.render_path)
-        
+
         if self.is_frame_sequence_render(vcodec) == True and vformat == None:
             # Frame sequence render
             consumer = renderconsumer.get_img_seq_render_consumer_codec_ext(render_item.render_path,
